@@ -26,6 +26,7 @@
 #include <QObject>
 
 #include <pulse/pulseaudio.h>
+#include <pulse/ext-stream-restore.h>
 
 namespace QPulseAudio
 {
@@ -37,6 +38,7 @@ class Sink;
 class SinkInput;
 class Source;
 class SourceOutput;
+class StreamRestore;
 
 /**
  * @see MapBase
@@ -46,10 +48,15 @@ class SourceOutput;
 class Q_DECL_EXPORT MapBaseQObject : public QObject
 {
     Q_OBJECT
+
+public:
+    virtual int count() const = 0;
+    virtual QObject *objectAt(int index) const = 0;
+    virtual int indexOfObject(QObject *object) const = 0;
+
 signals:
-    void added(quint32 index);
-    void updated(quint32 index);
-    void removed(quint32 index);
+    void added(int index);
+    void removed(int index);
 };
 
 /**
@@ -66,37 +73,28 @@ public:
 
     const QMap<quint32, Type *> &data() const { return m_data; }
 
-    int modelIndexForQObject(QObject *qobject) const
+    int count() const Q_DECL_OVERRIDE
     {
-        Type *t = qobject_cast<Type *>(qobject);
-        if (!t)
-            return -1;
-        int key = m_data.key(t, -1);
-        if (key == -1)
-            return -1;
-        return m_data.keys().indexOf(key);
+        return m_data.count();
     }
 
-    /**
-    * @param dataIndex index in the data model
-    * @return -1 on invalid index, otherwise PA index
-    */
-    qint64 dataIndexToPaIndex(int dataIndex) const
+    int indexOfObject(QObject *object) const Q_DECL_OVERRIDE
     {
-        auto list = m_data.values();
-        qCDebug(PLASMAPA) <<  Q_FUNC_INFO << list.length() << dataIndex;
-        if (list.length() <= dataIndex) {
-            return -1;
+        int index = 0;
+        QMapIterator<quint32, Type *> it(m_data);
+        while (it.hasNext()) {
+            it.next();
+            if (it.value() == object) {
+                return index;
+            }
+            index++;
         }
-        qCDebug(PLASMAPA) << "  " << list.at(dataIndex)->index();
-        qCDebug(PLASMAPA) << "  " << list.at(dataIndex)->name();
-        return list.at(dataIndex)->index();
+        return -1;
     }
 
-    int paIndexToDataIndex(quint32 index) const
+    QObject *objectAt(int index) const Q_DECL_OVERRIDE
     {
-        qCDebug(PLASMAPA) << m_data.keys() << index;
-        return m_data.keys().indexOf(index);
+        return (m_data.constBegin() + index).value();
     }
 
     void reset()
@@ -107,6 +105,17 @@ public:
         m_pendingRemovals.clear();
     }
 
+    void insert(Type *object)
+    {
+        Q_ASSERT(!m_data.contains(object->index()));
+
+        m_data.insert(object->index(), object);
+
+        const int modelIndex = m_data.keys().indexOf(object->index());
+        Q_ASSERT(modelIndex >= 0);
+        emit added(modelIndex);
+    }
+
     // Context is passed in as parent because context needs to include the maps
     // so we'd cause a circular dep if we were to try to use the instance here.
     // Plus that's weird separation anyway.
@@ -128,13 +137,10 @@ public:
         obj->update(info);
         m_data.insert(info->index, obj);
 
-        const int modelIndex = m_data.keys().indexOf(info->index);
-        Q_ASSERT(modelIndex >= 0);
-
         if (isNew) {
+            const int modelIndex = m_data.keys().indexOf(info->index);
+            Q_ASSERT(modelIndex >= 0);
             emit added(modelIndex);
-        } else {
-            emit updated(modelIndex);
         }
     }
 
@@ -144,7 +150,7 @@ public:
             m_pendingRemovals.insert(index);
         } else {
             const int modelIndex = m_data.keys().indexOf(index);
-            m_data.take(index)->deleteLater();
+            delete m_data.take(index);
             emit removed(modelIndex);
         }
     }
@@ -160,6 +166,7 @@ typedef MapBase<Source, pa_source_info> SourceMap;
 typedef MapBase<SourceOutput, pa_source_output_info> SourceOutputMap;
 typedef MapBase<Client, pa_client_info> ClientMap;
 typedef MapBase<Card, pa_card_info> CardMap;
+typedef MapBase<StreamRestore, pa_ext_stream_restore_info> StreamRestoreMap;
 
 } // QPulseAudio
 
